#include <fstream>
#include <iostream>
#include <string>
#include <cstring>

#include "pnm.h"

std::istream& operator>>(std::istream& s, pnm_t& rhs)
{
	char	c;

  // read 'P' if there
  c = s.get();
  if(s.eof()) {
    std::cerr << "Warning: EOF / read error reading magic number" << std::endl;
    return s;
  }
  if(c != 'P') {
    std::cerr << "Warning: Unknown image file type, not PNM" << std::endl;
    return s;
  }
  // get image type, eating whitespace at EOL
  s >> rhs.ptype >> std::ws;
  if(s.eof()) {
    std::cerr << "Warning: EOF / read error reading magic number" << std::endl;
    return s;
  }

  // determine bytes per pixel
  switch(rhs.ptype) {
    case 5: rhs.bpp = 1; break;
    case 6: rhs.bpp = 3; break;
    case 7: rhs.bpp = 4; break;
  }

  // eat comment lines
  while((c = s.peek()) == '#') do { s.get(c); } while(c != '\n');

  // get w,h, eating ws at EOL
  s >> rhs.w >> rhs.h >> std::ws;

  // get max (color), eating ws at EOL
  s >> rhs.max >> std::ws;

  // allocate and read the data
  rhs.img = new char [rhs.bpp * rhs.w * rhs.h];
  s.read(rhs.img,rhs.bpp*rhs.w*rhs.h);

  return s;
}

std::ostream& operator<<(std::ostream& s, const pnm_t& rhs)
{
  s << 'P' << rhs.ptype << std::endl;
  s << "# Stored by pnm_t class." << std::endl;
  s << rhs.w << " " << rhs.h << std::endl << rhs.max << std::endl;

  s.write(rhs.img,rhs.bpp*rhs.w*rhs.h);

  return s;
}

const pnm_t& pnm_t::operator=(const pnm_t& rhs)
{
  if(this != &rhs) {	// standard alias test
    // copy all elements of rhs into this except for name
    ptype = rhs.ptype;
    max = rhs.max;
    w = rhs.w;  h = rhs.h;
    bpp = rhs.bpp;
    if(img) delete img;
    img = new char [bpp * w * h];
    memcpy(img,rhs.img,bpp*w*h);
  }
  return *this;
}

// copy constructor
pnm_t::pnm_t(const pnm_t& rhs)
{
  // copy constructor creates the pnm_t object given a constant reference to
  // another pnm_t object (result is a copy of the other object)
  // copy all elements of rhs into this except for name
  ptype = rhs.ptype;
  max = rhs.max;
  w = rhs.w;  h = rhs.h;
  bpp = rhs.bpp;
  img = new char [rhs.bpp * rhs.w * rhs.h];

  memcpy(img,rhs.img,rhs.bpp*rhs.w*rhs.h);
}
