#include <fstream>
#include <iostream>
#include <string>
#include <cstring>

#include "pgm.h"
#include "ppm.h"

std::istream& operator>>(std::istream& s, pgm_t& rhs)
{
	char	c, *data;

  // read 'P' if there
  c = s.get();
  if(s.eof()) {
    std::cerr << "Warning: EOF / read error reading magic number" << std::endl;
    return s;
  }
  if(c != 'P') {
    std::cerr << "Warning: Unknown image file type, not PNM" << std::endl;
    return s;
  }
  // get image type, eating whitespace at EOL
  s >> rhs.ptype >> std::ws;
  if(s.eof()) {
    std::cerr << "Warning: EOF / read error reading magic number" << std::endl;
    return s;
  }

  // determine bytes per pixel
  switch(rhs.ptype) {
    case 5: rhs.bpp = 1; break;
    case 6: rhs.bpp = 3; break;
    case 7: rhs.bpp = 4; break;
  }

  // eat comment lines
  while((c = s.peek()) == '#') do { s.get(c); } while(c != '\n');

  // get w,h, eating ws at EOL
  s >> rhs.w >> rhs.h >> std::ws;

  // get max (color), eating ws at EOL
  s >> rhs.max >> std::ws;

  // allocate and read the data
  rhs.pix = new float [rhs.w * rhs.h];

  // read uchar data
//data = new char [rhs.bpp * rhs.w * rhs.h];
//s.read(data,rhs.bpp*rhs.w*rhs.h);
  data = new char [rhs.w * rhs.h];
  s.read(data,rhs.w*rhs.h);

  if(rhs.pix) {
    for(int i=0;i<rhs.h;i++) {
      for(int j=0;j<rhs.w;j++) {
        rhs.pix[i*rhs.w + j] = (float)data[i*rhs.w + j]/(float)rhs.max;
      }
    }
  }

  delete [] data;

  return s;
}

std::ostream& operator<<(std::ostream& s, const pgm_t& rhs)
{
	char	c, *data = new char [rhs.w * rhs.h];

  for(int i=0;i<rhs.h;i++) {
    for(int j=0;j<rhs.w;j++) {
      data[i*rhs.w + j] = (char)(rhs.pix[i*rhs.w + j]*(float)rhs.max);
    }
  }

  s << 'P' << rhs.ptype << std::endl;
  s << "# Stored by pgm_t class." << std::endl;
  s << rhs.w << " " << rhs.h << std::endl << rhs.max << std::endl;

//s.write(rhs.pix,rhs.bpp*rhs.w*rhs.h);
  s.write(data,rhs.w*rhs.h);

  delete [] data;

  return s;
}

const pgm_t& pgm_t::operator=(const pgm_t& rhs)
{
  if(this != &rhs) {	// standard alias test
    // copy all elements of rhs into this
    ptype = rhs.ptype;
    max = rhs.max;
    w = rhs.w;  h = rhs.h;
    bpp = rhs.bpp;
    if(pix) delete [] pix;
    pix = new float [w * h];
    memcpy(pix,rhs.pix,w*h*sizeof(float));
  }
  return *this;
}

pgm_t::pgm_t(const pgm_t& rhs)
{
  // copy constructor creates the pgm_t object given a constant reference to
  // another pgm_t object (result is a copy of the other object)
  // copy all elements of rhs into this except for name
  ptype = rhs.ptype;
  max = rhs.max;
  w = rhs.w;  h = rhs.h;
  bpp = rhs.bpp;
  pix = new float [rhs.w * rhs.h];

  memcpy(pix,rhs.pix,rhs.w*rhs.h*sizeof(float));
}

void pgm_t::fromppm(const ppm_t& ppm)
{
	float	r, g, b;
	float	*p6, *p5;

  ptype = 5;
  w = ppm.w;
  h = ppm.h;
  bpp = 1;
  max = ppm.max;

//if(pix) { delete [] pix; } // bug...why? because pix could point to garbage

//pix = new char [bpp * w * h];
  pix = new float [w * h];

  // set up float* pointers for some tasty pointer math
  p6 = ppm.pix; p5 = pix;

  // loop through color image, convert to float, then do luminance conversion
  for(int i=0;i<w*h;i++) {
    // convert char bytes to float, remember to normalize
    r = *p6++;
    g = *p6++;
    b = *p6++;

    // note that g variable serves double duty here: once as green, then grey
    *p5++ = 0.21 * r + 0.72 * g + 0.07 * b;
  }
}
