#include <fstream>
#include <iostream>
#include <string>
#include <cstring>

#include "pgm.h"
#include "ppm.h"

std::istream& operator>>(std::istream& s, ppm_t& rhs)
{
	char		c;
	char	p;

  // read 'P' if there
  c = s.get();
  if(s.eof()) {
    std::cerr << "Warning: EOF / read error reading magic number" << std::endl;
    return s;
  }
  if(c != 'P') {
    std::cerr << "Warning: Unknown image file type, not PNM" << std::endl;
    return s;
  }
  // get image type, eating whitespace at EOL
  s >> rhs.ptype >> std::ws;
  if(s.eof()) {
    std::cerr << "Warning: EOF / read error reading magic number" << std::endl;
    return s;
  }

  // determine bytes per pixel
  switch(rhs.ptype) {
    case 5: rhs.bpp = 1; break;
    case 6: rhs.bpp = 3; break;
    case 7: rhs.bpp = 4; break;
  }

  // eat comment lines
  while((c = s.peek()) == '#') do { s.get(c); } while(c != '\n');

  // get w,h, eating ws at EOL
  s >> rhs.w >> rhs.h >> std::ws;

  // get max (color), eating ws at EOL
  s >> rhs.max >> std::ws;

  // allocate and read the data
  rhs.pix = new float [rhs.bpp * rhs.w * rhs.h];

  // read uchar data
//data = new char [rhs.bpp * rhs.w * rhs.h];
//s.read(data,rhs.bpp*rhs.w*rhs.h);
//data = new char [rhs.w * rhs.h];
//s.read(data,rhs.w*rhs.h);

  if(rhs.pix) {
    for(int b=0;b<rhs.bpp;b++) {
      for(int i=0;i<rhs.h;i++) {
        for(int j=0;j<rhs.w;j++) {
          s.read(&p,1);
          rhs.pix[b*(rhs.w*rhs.h) + i*rhs.w + j] = (float)p/(float)rhs.max;
        }
      }
    }
  }

  return s;
}

std::ostream& operator<<(std::ostream& s, const ppm_t& rhs)
{
	char	p;

  s << 'P' << rhs.ptype << std::endl;
  s << "# Stored by ppm_t class." << std::endl;
  s << rhs.w << " " << rhs.h << std::endl << rhs.max << std::endl;

//s.write(rhs.pix,rhs.bpp*rhs.w*rhs.h);
  for(int b=0;b<rhs.bpp;b++) {
    for(int i=0;i<rhs.h;i++) {
      for(int j=0;j<rhs.w;j++) {
        p = (char)(rhs.pix[b*(rhs.w * rhs.h) + i*rhs.w + j]*(float)rhs.max);
        s.write(&p,1);
      }
    }
  }

  return s;
}
